@@ -1,5 +1,5 @@
 package Data::Schema::Type::Either;
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 
 # ABSTRACT: Type handler for 'either' type
@@ -10,15 +10,15 @@ extends 'Data::Schema::Type::Base';
 with 'Data::Schema::Type::Scalar';
 
 
+sub chkarg_attr_of {
+    my ($self, $arg, $name) = @_;
+    $self->chkarg_r_array_of_schema($arg, $name);
+}
+
 sub handle_attr_of {
     my ($self, $data, $arg) = @_;
     my $ds = $self->validator;
 
-    unless (ref($arg) eq 'ARRAY' && @$arg) {
-        $ds->schema_error("`of` attribute must be array with elements");
-        return;
-    }
-
     my $success;
     for my $i (0..@$arg-1) {
         $ds->save_validation_state();
@@ -37,11 +37,6 @@ sub emitpl_attr_of {
     my $ds = $self->validator;
     my $perl = '';
 
-    unless (ref($arg) eq 'ARRAY' && @$arg) {
-        $ds->schema_error("arg of `of` must be array with elements");
-        return;
-    }
-
     my @schemas;
     for my $i (0..@$arg-1) {
 	my ($code, $csubname) = $ds->emitpls_sub($arg->[$i]);
@@ -54,7 +49,7 @@ sub emitpl_attr_of {
     $perl .= $ds->emitpl_my('$found');
     $perl .= '$found = 0;'."\n";
     $perl .= 'for my $i (0..@schemas-1) {'."\n";
-    $perl .= '    my ($suberrors) = $schemas[$i]($data);'."\n";
+    $perl .= '    my ($suberrors, $subwarnings) = $schemas[$i]($data);'."\n";
     $perl .= '    if (!@$suberrors) { $found++; last }'."\n";
     $perl .= "}\n";
     $perl .= 'if (!$found) { '.$ds->emitpl_data_error("data does not validate to any alternatives")." }\n";
@@ -101,7 +96,7 @@ Data::Schema::Type::Either - Type handler for 'either' type
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =head1 SYNOPSIS
 
@@ -109,11 +104,11 @@ version 0.12
 
 =head1 DESCRIPTION
 
+Aliases: or, any
+
 'Either' is not really an actual data type, but a way to validate whether a
 value validates to any one of the specified schemas.
 
-Synonym: or, any
-
 Example schema (in YAML syntax):
 
  - any
